@@ -1,8 +1,8 @@
-import math
-import collections
+from copy import copy
+import itertools
 
-from visidata import *
-from visidata.pivot import PivotSheet
+from visidata import vd, asyncthread, vlen, VisiData, Column, AttrColumn, Sheet, ColumnsSheet, ENTER
+from visidata.pivot import PivotSheet, PivotGroupRow
 
 
 vd.option('disp_histogram', '*', 'histogram element character')
@@ -19,6 +19,12 @@ def valueNames(vd, discrete_vals, numeric_vals):
 
     return '+'.join(ret)
 
+class HistogramColumn(Column):
+    def calcValue(col, row):
+        histogram = col.sheet.options.disp_histogram
+        histolen = col.sheet.options.disp_histolen
+        return histogram*(histolen*len(row.sourcerows)//col.sheet.largest)
+
 
 class FreqTableSheet(PivotSheet):
     'Generate frequency-table sheet on currently selected column.'
@@ -47,10 +53,13 @@ class FreqTableSheet(PivotSheet):
 
         # add default bonus columns
         for c in [
-                    ColumnAttr('count', 'sourcerows', type=vlen),
-                    Column('percent', type=float, getter=lambda col,row: len(row.sourcerows)*100/col.sheet.source.nRows),
-                    Column('histogram', type=str, getter=lambda col,row: options.disp_histogram*(options.disp_histolen*len(row.sourcerows)//col.sheet.largest), width=options.disp_histolen+2),
-                    ]:
+            AttrColumn('count', 'sourcerows', type=vlen),
+            Column('percent', type=float, getter=lambda col,row: len(row.sourcerows)*100/col.sheet.source.nRows),
+        ]:
+            self.addColumn(c)
+
+        if self.options.disp_histolen and self.options.disp_histogram:
+            c = HistogramColumn('histogram', type=str, width=self.options.disp_histolen+2)
             self.addColumn(c)
 
         # two more threads
@@ -74,6 +83,12 @@ class FreqTableSheet(PivotSheet):
             return vs
         vd.warning("no source rows")
 
+    def openRows(self, rows):
+        vs = copy(self.source)
+        vs.name += "_several"
+        vs.rows = list(itertools.chain.from_iterable(row.sourcerows for row in rows))
+        return vs
+
     def openCell(self, col, row):
         return Sheet.openCell(self, col, row)
 
@@ -93,7 +108,7 @@ ColumnsSheet.addCommand(ENTER, 'freq-row', 'vd.push(FreqTableSheet(source[0], cu
 vd.addMenuItem('Data', 'Frequency table', 'current row', 'freq-row')
 
 FreqTableSheet.addCommand('gu', 'unselect-rows', 'unselect(selectedRows)', 'unselect all source rows grouped in current row')
-FreqTableSheet.addCommand('g'+ENTER, 'dive-rows', 'vs = copy(source); vs.name += "_several"; vs.rows=list(itertools.chain.from_iterable(row.sourcerows for row in selectedRows)); vd.push(vs)', 'open copy of source sheet with rows that are grouped in selected rows')
+FreqTableSheet.addCommand('g'+ENTER, 'dive-selected', 'vd.push(openRows(selectedRows))', 'open copy of source sheet with rows that are grouped in selected rows')
 
 vd.addGlobals({
     'FreqTableSheet': FreqTableSheet,
